/********************************************************
generate.js - version 3.1

This file is a part of SmoothVideo Project (SVP) package.
See License_eng.txt for licensing.

Description: script for building Avisynth input script.

:     Avisynth.
********************************************************/

//helper functions to read from multiply configs
function value(name,def)
{
	name=name.toLowerCase();
	if((res=confSettings.main[name]) !== undefined) return res;
	if((res=confSVPmgr.main[name]) !== undefined) return res;
	if((res=confDefault.main[name]) !== undefined) return res;
	return def;
}

function valueProfile(name)
{
	name=name.toLowerCase();
	if((res=confProfile.main[name]) !== undefined) return res;
	if((res=confProfileDefaults.main[name]) !== undefined) return res;
}

//read all the configuration files
mediaInfo=SVP.ReadObjectFromIni(SVP.iniDir+"MediaInfo.ini");
confSettings=SVP.ReadObjectFromIni(SVP.iniDir+"settings.ini");
confSVPmgr=SVP.ReadObjectFromIni(SVP.iniDir+"SVPmgr.ini");
confDefault=SVP.ReadObjectFromIni(SVP.iniDir+"default.ini");
confProfile=SVP.ReadObjectFromIni(SVP.extIniDir+mediaInfo.main.optprofile);
confProfileDefaults=SVP.ReadObjectFromIni(SVP.iniDir+"MSmoothFps.ini");

//global variables
var gpu=(value("svp_libflowgpu",0)!=0);
var encode = (WScript.Arguments.length>0 && WScript.Arguments(0)=="encode");
var demoMode=(value("ExDemo",0)==1) && !encode;

//parameters for SVSuper
function defineSuper()
{
	var levels={};
	pel=valueProfile("ExPel");
	if (pel>0 && pel!=2) levels.pel=pel;
	if (pel==0) levels.pel=1;
	levels.scale={};
	levels.scale.up=(gpu ? 0:2);
	levels.padding={};
	levels.gpu=(gpu? 1:0);
	if (pel<2) levels.full=false;

	return levels;
}

//parameters for SVAnalyse
function defineAnalyse()
{
	var analyse={};
	if(valueProfile("ExAlgo")==1)
		analyse.vectors=2;

	analyse.block={};
	blksz=valueProfile("ExBlockSize").split(/[:x]/);
	blk_w=parseInt(blksz[0]);
	blk_h=parseInt(blksz[1]);
	overlap=parseInt(blksz[2]);

	overlappx=(blk_w<blk_h ? blk_w:blk_h);
	if(!gpu && overlappx<16 && overlap==1)
		overlap=0;
	switch(overlap)
	{
	case 1: overlappx=overlappx/8; break;
	case 2: overlappx=overlappx/4; break;
	case 3: overlappx=overlappx/2; break;
	default: overlappx=0; break;
	}

	if(blk_w!=16) analyse.block.w=blk_w;
	if(blk_h!=16) analyse.block.h=blk_h;
	if(overlap!=2) analyse.block.overlap=overlap;

	analyse.main={};
	analyse.main.search={};
	analyse.main.search.coarse={};

	search=valueProfile("TypeDist").split(":");
	if(search[0]=="Hex") stype=2;
	else if (search[0]=="UMH") stype=3;
	else if (search[0]=="Exh") stype=4;

	satd=(search[2]=="SATD");

	if(stype!=4)
		analyse.main.search.coarse.type=stype;
	if(parseInt(search[1])!=0)
		analyse.main.search.coarse.distance=parseInt(search[1]);
	if(!satd)
		analyse.main.search.coarse.satd=satd;
	if(pel>0)
	{
		if(stype!=4)
			analyse.main.search.type=stype;
	}
	else analyse.main.search.distance=0;

	analyse.main.search.coarse.bad={};
	badsad=valueProfile("Badsad").split(":");
	if(badsad[0]=="bad")
		badsad[0]=blk_h*blk_w*8;
	if(badsad[0]!=1000)
		analyse.main.search.coarse.bad.sad=parseInt(badsad[0]);
	if(badsad[1]!=-24)
		analyse.main.search.coarse.bad.range=parseInt(badsad[1]);

	analyse.main.penalty={}

	analyse.refine=[];

	count=0;
	recalc=valueProfile("ExRecalc").split(":");
	thsad=parseInt(recalc[0]);
	newsize=parseInt(recalc[1]);
	if(!gpu) overlappx/=2;

	if(newsize==0) newsize=blk_h/2;
	if(newsize<blk_h && thsad!=0 && overlappx!=1)
	{
		analyse.refine[0]={};
		analyse.refine[0].thsad=thsad;
		analyse.refine[0].search={};
		analyse.refine[0].penalty={};
		count++;
		if (newsize*2<blk_h && overlappx!=2)
		{
			analyse.refine[count]={};
			analyse.refine[count].thsad=thsad;
			analyse.refine[count].search={};
			analyse.refine[count].penalty={};
			count++;
			if (newsize*4<blk_h && overlappx!=4)
			{
				analyse.refine[count]={};
				analyse.refine[count].thsad=thsad;
				analyse.refine[count].search={};
				analyse.refine[count].penalty={};
			}
		}
	}
	return analyse;
}

//parameters for black border lighting
function defineLights()
{
	var lights={};
	if(demoMode) return lights;

	borderlight=value("Borderlight","");
	if(borderlight=="None") return lights;
	borderlightType=value("BorderlightType","");
	if(borderlightType=="CustomLight")
		borderlightType=value("CustomBorderlightType","");
	if(borderlightType!=="" && value("ExDemo",0)==0)
	{
		if(borderlight=="MON" || borderlight=="Full")
			lights.aspect=Math.round(parseFloat(mediaInfo.main.monwidth)*1000/parseFloat(mediaInfo.main.monheight))/1000;

		if(borderlightType!=="Default")
		{
			light=borderlightType.split(":");
			if(light.length)
				lights.lights=parseInt(light[0]);
			if(light.length>1)
				lights.border=parseInt(light[1]);
			if(light.length>2)
				lights.length=parseInt(light[2]);
			if(light.length>3)
				lights.cell=parseFloat(light[3]);
		}
	}
	if(borderlight=="Zoom" || borderlight=="Full")
	{
		lightszoom=parseFloat(mediaInfo.main.lightszoom);
		if(lightszoom!=0) lights.zoom=lightszoom;
	}
	return lights;
}

function apply_autocrop(smooth)
{	if(value("autocrop", 0)==1 && mediaInfo.main.autocrop=="" && !encode)
	{
		smooth.debug={};
		smooth.debug.fields=true;
	}
}

//parameters for SVSmoothFps
function defineSmooth(mode)
{
	var smooth={};

	//gpuID
	gpuID = value("svp_libflowgpu", 0);
	if (gpuID>1)
		smooth.gpuid=gpuID;

	marks = value("GPU_"+(gpuID>1 ? gpuID : 11)+"_mark", 1000);
	k = mediaInfo.main.reswidth*mediaInfo.main.resheight*mediaInfo.main.fres/1920/1080/60;
	if(marks<250*k)
		smooth.cubic=0;
	if(marks<350*k)
		smooth.linear=false;

	smooth.rate={}
	smooth.rate.num=mediaInfo.main.num;
	smooth.rate.den=mediaInfo.main.den;
	if(stereoMode!=0 && mediaInfo.main.fres<73)
		smooth.rate.num=smooth.rate.num*2;

	algo=valueProfile("ExAlgo");
	block=false;
	if(algo==="11b")
	{
		block=true;
		algo=11;
	}
	smooth.algo=algo;


	if(block && !gpu)
		smooth.block=true;

	smooth.mask={};
	sadml=valueProfile("ExSadml");
	if(sadml!=0)
		smooth.mask.area=sadml;
	if(smooth.algo>=21)
		smooth.mask.cover=80;

	smooth.scene={};
	if(valueProfile("ExBlend")=="true")
		smooth.scene.blend = true;

	smooth.scene.limits={};
	modeN=parseInt(mode);
	if(mode=="1.5m")
	{
		modeN=3;
		smooth.scene.limits.m1=0;
		smooth.scene.limits.m2=0;
	}

	if(modeN==2 && mediaInfo.main.num/mediaInfo.main.den<2)
		modeN=1;
	if(modeN>0 && mediaInfo.main.num/mediaInfo.main.den<1)
		modeN=0;
	if(modeN!=3)
		smooth.scene.mode=modeN;

	smooth.light=defineLights();

	apply_autocrop(smooth);

	return smooth;
}

//handle stereo modes
function checkStereo()
{
	stereoMode=value("StereoMode",0);
	dwnResize=""+valueProfile("ExDwnResize");
	relativeResize=(parseInt(dwnResize)>1);
	fixedWidth=fixedHeight=false;
	if(!relativeResize)
	{
		fixedWidth=(dwnResize.indexOf("w")==0);
		fixedHeight=(dwnResize.indexOf("h")==0);
	}
	stereoLeftSelection="";
	stereoRightSelection="";
	if(stereoMode!=0)
	{
		stereoLeftSelection=".SelectEven()";
		stereoRightSelection=".SelectEven()";
		stereoFramesOrder=value("StereoFramesOrder",1);
		if(stereoFramesOrder==1) stereoRightSelection=".SelectOdd()";
		if(stereoFramesOrder==2) stereoLeftSelection=".SelectOdd()";
	}
}

checkStereo();

//create paramters objects
var levels,analyse,smooth;

doSmooth=(confProfile.main.exmethod.indexOf("MSmoothFps")==0);
demoMode = demoMode && doSmooth;

if(doSmooth)
{
	mode=confProfile.main.exmethod.split("_")[1];
	levels=defineSuper();
	analyse=defineAnalyse();
	smooth=defineSmooth(mode);

	override=SVP.ReadFile(SVP.currentDir+"override.js");
	if(override.length)
		eval(override);
}
else
{
	levels={};
	analyse={};
	smooth={rate:{num:1}};
	smooth.light=defineLights();
	apply_autocrop(smooth);
}

levels.rc=!encode;

//number of threads
threads=value("ExThreads",0);
if(!threads)
{
	threadsSys=parseInt(SVP.shell.ExpandEnvironmentStrings("%NUMBER_OF_PROCESSORS%"));
	threads=(gpu ? Math.round(threadsSys*1.823) : threadsSys+1);
}
if(mediaInfo.main.num==1 && mediaInfo.main.den==1 && threads>5)
	threads=5;

//generate AVS script file
appdatadir = SVP.appDataDir;
if (WScript.Arguments.length>1) appdatadir = WScript.Arguments(1);
if (!encode)
	avs=SVP.CreateFile(appdatadir+"AVS\\ffdshow.avs")
else
	avs=SVP.CreateFile(appdatadir+"AVS\\encode.avs")

avs.WriteLine("# This script was generated by SmoothVideo Project (SVP) Manager.");
avs.WriteLine("# Check http://www.svp-team.com for more details.");
avs.WriteLine("");

avs.WriteLine("SetMemoryMax("+value("AVSmem",1024)+")");
avs.WriteLine("");

avs.WriteLine("global svp_scheduler=true");
avs.WriteLine('global threads='+threads);
avs.WriteLine("global svp_cache_fwd=threads+2");
avs.WriteLine("");

avs.WriteLine('LoadPlugin("'+SVP.currentDir+'plugins\\svpflow1.dll")');
avs.WriteLine('LoadPlugin("'+SVP.currentDir+'plugins\\svpflow2.dll")');
avs.WriteLine("");

if(threads>1)
	avs.WriteLine('SetMTMode(3,threads)');

if(!encode)
	avs.WriteLine("ffdShow_source()")
else if (mediaInfo.main.avssrc=='DirectShowSource')
	avs.WriteLine('DirectShowSource("'+mediaInfo.main.filepath+'\\'+
				mediaInfo.main.filename+'", fps='+
				mediaInfo.main.fps+
				', convertfps=true).ConvertToYV12()');

AddAssert = (confSVPmgr.main.AddAssert==1);
if(!encode && AddAssert)
	avs.WriteLine('assert(width=='+mediaInfo.main.xish+' && height=='+mediaInfo.main.yish+','+'"Frame size "+string(width)+"x"+string(height)+" instead of '+mediaInfo.main.xish+'x'+mediaInfo.main.yish+'")');

if(threads>1)
	avs.WriteLine('SetMTMode(2)');
avs.WriteLine("");

if(value("DropEven",0) == 1)
{
	smooth.rate.num*=2;
	avs.WriteLine("SelectEvery(2,0)");
	avs.WriteLine("");
}

//crop input
cropParams="";
if(mediaInfo.main.cropparams!="0:0:0:0")
{
	params=mediaInfo.main.cropparams.split(":");
	cropParams=params[0]+","+params[1]+",-"+params[2]+",-"+params[3];
}
avs.WriteLine('global crop_params="'+cropParams+'"');

//resize input
resizeString="";
if(	valueProfile("ExDwnResize")!==0 &&
	(mediaInfo.main.cbhwidth != mediaInfo.main.reswidth ||
	mediaInfo.main.cbhheight != mediaInfo.main.resheight))
{
	//downsize
	if(	mediaInfo.main.reswidth>mediaInfo.main.cbhwidth ||
		mediaInfo.main.resheight>mediaInfo.main.cbhheight)
		resizeString="BicubicResize("+mediaInfo.main.reswidth+", "+mediaInfo.main.resheight+", b=0, c=0.75)";
	else //upsize
		resizeString="LanczosResize("+mediaInfo.main.reswidth+","+mediaInfo.main.resheight+")";
}
avs.WriteLine('global resize_string="'+resizeString+'"');

avs.WriteLine('global super_params="'+SVP.ObjectToString(levels)+'"');
avs.WriteLine('global analyse_params="'+SVP.ObjectToString(analyse)+'"');
avs.WriteLine('global smoothfps_params="'+SVP.ObjectToString(smooth)+'"');
avs.WriteLine("");
avs.WriteLine("global demo_mode="+(demoMode?1:0));
avs.WriteLine("stereo_type="+stereoMode);
avs.WriteLine('stereo_left_selection="'+stereoLeftSelection+'"');
avs.WriteLine('stereo_right_selection="'+stereoRightSelection+'"');

if(demoMode)
{
	avs.WriteLine("");
	avs.WriteLine('function demo(clip src,clip smooth)');
	avs.WriteLine('{');
	avs.WriteLine('\tsrcd = src.crop(0,0,-Int(src.width/4)*2-2,0)');
	avs.WriteLine('\tsrcd = srcd.ChangeFPS(FramerateNumerator(src)*'+mediaInfo.main.num+', FramerateDenominator(src)*'+mediaInfo.main.den+')');
	avs.WriteLine('\tres = smooth.crop(Int(src.width/4)*2,0,0,0)');
	avs.WriteLine('\treturn StackHorizontal(srcd.Subtitle("Source", align=2),srcd.BlankClip(width=2),res.Subtitle("Smooth", align=2))');
	avs.WriteLine('}');
	avs.WriteLine('');
}

avs.WriteLine("");
avs.write(SVP.ReadFile(SVP.avsDir+"MSmoothFps.avs"));

if(!encode && threads>1)
	avs.WriteLine("distributor()");

avs.Close();
